package com.perforce.hws.core.p4base;

import com.perforce.p4java.impl.mapbased.rpc.RpcServer;
import com.perforce.p4java.impl.mapbased.server.Server;
import com.perforce.p4java.server.IServer;

import java.util.function.Supplier;

/**
 * Use the CloseableServer instances to use IServer methods in a
 * try-with-resources block.
 * <p>
 * This is currently used in our automation layer where we need to frequently
 * create and close connections explicitly.
 */
public class CloseableServer implements AutoCloseable, Supplier<IServer> {

    private IServer server;
    private CloseableServerProvider provider;

    public CloseableServer(CloseableServerProvider provider, IServer server) {
        this.provider = provider;
        this.server = server;
    }

    /**
     * This means "release" to our pooling mechanism.
     *
     * @throws P4JavaRuntimeException
     */
    @Override
    public void close() throws P4JavaRuntimeException {
        provider.release(this);
    }

    @Override
    public IServer get() {
        return server;
    }

    public IServer getServer() {
        return server;
    }

    public void setServer(IServer server) {
        this.server = server;
    }

    public void setClientName(String clientName) {
        ((Server) this.server).setClientName(clientName);
    }

    public void setApiLevel(int level) {
        ((RpcServer) this.server).setClientApiLevel(level);
    }
}
